package org.deken.game.animation;

import java.awt.Graphics2D;
import java.awt.Image;

import org.deken.game.Updateable;

/**
 * <p>Title: Animating</p>
 *
 * <p>Description: Interface for AnimationStrips. This interface supplies
 * all the needed methods to connected it to either an Actor or Decor object.</p>
 * <p>Implementing this interface will allow you to create your own imaging
 * sequence for a character.</p>
 *
 * <p>Copyright: Copyright (c) 2005</p>
 *
 * <p>@author Gary Deken</p>
 * @version 1.0
 */
public interface Animation extends Updateable, Cloneable {
   /**
    * addFrame
    *
    * <p>This method is for adding an AnimationFrame to the animation strip.</p>
    *
    * @param image Image
    * @param duration long
    */
   public void addFrame(Image image, long duration);

   /**
    * reset
    *
    * <p>This resets the animation strip back to its default (or initial start)
    * frame. This is useful for starting a game piece back to the begining after
    * it stops.</p>
    */
   public void reset();

   /**
    * update
    *
    * <p>This method takes the elapse time, and calculates the new current frame
    * of the animation.</p>
    *
    * @param elapseTime long
    */
   public void update(long elapseTime);

   /**
    * getImage
    *
    * <p>This method returns the current frame of the animation. When it is used
    * with the update method, it will return each image in the sequence as it is
    * updated.</p>
    *
    * @return Image
    */
   public Image getImage();

   /**
    * draw
    * Draws the current frame or shape of the animation at the set x and y location.
    * It also performs any offsets that are needed for the image.
    * 
    * @param x
    * @param y
    */
   public void draw(int x, int y);

   /**
    * getFrame
    *
    * <p>This method returns an Image at a given frame location. NOTE: this is
    * useful for getting a static (stand still) image for a game piece.</p>
    *
    * @param i int
    * @return Image
    */
   public Image getFrame(int i);

   /**
    * getHeight
    *
    * <p>This returns the height of the animation strip.</p>
    *
    * @return int
    */
   public int getHeight();

   /**
    * getWidth
    *
    * <p>This returns the width of the animation strip.</p>
    *
    * @return int
    */
   public int getWidth();

   /**
    * setOffsets
    * Sets the offset of the image, so that it displays ideally on the screen.
    * This should default to zero.
    * @param x
    * @param y
    */
   public void setOffsets(int x, int y);

   /**
    * isAnimating
    *
    * <p>True if this animation needs to be updated regularly. False if there is
    * no animation to update, meaning getImage will return the same image.</p>
    *
    * @return boolean
    */
   public boolean isAnimating();

   /**
    * setName
    * Name is used for debugging.
    *
    * @param name String
    */
   public void setName(String name);

   /**
    * clone
    * Copies the animation.
    * @return Animation
    */
   public Object clone();
}
