package org.deken.game.animation;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;

import java.awt.Image;

import org.deken.exception.FactoryException;
import org.deken.game.document.DocumentAttributes;
import org.deken.images.ImageFactory;

/**
 * <p>Title: AnimationFactory</p>
 *
 * <p>Description: This class creates Animations from an images.</p>
 *
 * <p>Copyright: Copyright (c) 2005</p>
 *
 * <p>@author Gary Deken</p>
 *
 * @version 1.0
 */
public class AnimationFactory {
  private ImageFactory imageFactory;

  /**
   * AnimationFactory
   */
  public AnimationFactory() {
    imageFactory = ImageFactory.getInstance();
  }

  /**
   * createAnimation
   * Creates an Animation of the class passed in, by cutting the Image into
   * cells, and adding them to the Animation. The Animation Interface is
   * returned, and can be cast to the concrete implemenation. The Animation
   * class must have a constructor that takes an <code>Image</code> and a
   * <code>long</code>. If the Animation cannot be constructed, a
   * FactoryException is thrown.
   *
   * @param animationClass Class
   * @param imageToCut Image
   * @param cellWidth int
   * @param cellHeight int
   * @param numberOfCells int
   * @param durationInMilliseconds int
   * @return Animation
   * @throws FactoryException
   */
  public Animation createAnimation(Class animationClass, Image imageToCut,
                                   int cellWidth, int cellHeight, int numberOfCells,
                                   int durationInMilliseconds) throws
      FactoryException {
    Class clazz[] = new Class[] {Image.class, Long.TYPE};
    // get images from the imageToCut.
    Image[] imageAry = getImageArray(imageToCut, cellWidth,
                                     cellHeight, numberOfCells);
    Animation animate = createAnimation(animationClass, durationInMilliseconds, clazz, imageAry);
    return animate;
  }

  /**
   * createAnimation
   * Creates an Animation of the class passed in, by cutting the Image into
   * cells, and adding them to the Animation. The Animation Interface is
   * returned, and can be cast to the concrete implemenation. The Animation
   * class must have a constructor that takes an <code>Image</code> and a
   * <code>long</code>. If the Animation cannot be constructed, a
   * FactoryException is thrown.
   *
   * @param animationClass Class
   * @param imageToCut Image
   * @param cellWidth int
   * @param cellHeight int
   * @param durationInMilliseconds int
   * @return Animation
   * @throws FactoryException
   */
  public Animation createAnimation(Class animationClass, Image imageToCut,
                                   int cellWidth, int cellHeight,
                                   int durationInMilliseconds) throws
      FactoryException {
    Class clazz[] = new Class[] {Image.class, Long.TYPE};
    // get images from the imageToCut.
    Image[] imageAry = getImageArray(imageToCut, cellWidth,
                                     cellHeight);
    Animation animate = createAnimation(animationClass, durationInMilliseconds, clazz, imageAry);

    return animate;
  }

  /**
   * createAnimation
   * Creates an Animation of the class passed in, using the cells provided in
   * the Image array, and adding them to the Animation. The Animation Interface
   * is returned, and can be cast to the concrete implemenation. The Animation
   * class must have a constructor that takes an <code>Image</code> and a
   * <code>long</code>. If the Animation cannot be constructed, a
   * FactoryException is thrown.
   *
   * @param animationClass Class
   * @param durationInMilliseconds int
   * @param imageAry Image[]
   * @return Animation
   * @throws FactoryException
   */
  public Animation createAnimation(Class animationClass, Image[] imageAry, int durationInMilliseconds)
      throws FactoryException {
    Class clazz[] = new Class[] {Image.class, Long.TYPE};

    Animation animate = createAnimation(animationClass, durationInMilliseconds, clazz, imageAry);

    return animate;
  }

  /**
   * createAnimation
   * Creates an Animation of the class passed in, using a single image in the
   * Animation object, such as a StaticAnimationStrip. The Animation Interface
   * is returned, and can be cast to the concrete implemenation. The Animation
   * class must have a constructor that takes an <code>Image</code> and a
   * <code>long</code>. If the Animation cannot be constructed, a
   * FactoryException is thrown.
   *
   * @param animationClass Class
   * @param image Image
   * @return Animation
   * @throws FactoryException
   */
  public Animation createAnimation(Class animationClass, Image image) throws FactoryException {
    Class clazz[] = new Class[] {Image.class};
    Animation animate = null;
    // create the arguments for the constuctor.
    Object[] args = new Object[] {image};

    try {
      // get the constructor
      Constructor constr = animationClass.getConstructor(clazz);
      // instantiate the class.
      animate = (Animation) constr.newInstance(args);
    }
    catch (SecurityException ex) {
      throw new FactoryException("Not enough Security permission to get constructor " + animationClass.getName(), ex);
    }
    catch (NoSuchMethodException ex) {
      throw new FactoryException("The Animation class does not have the correct constructor./r/n" +
                                 "Needs a constructor that takes an Image and a long.", ex);
    }
    catch (InvocationTargetException ex) {
      throw new FactoryException("Unable to create a new Instance of " + animationClass.getName(), ex);
    }
    catch (IllegalArgumentException ex) {
      throw new FactoryException("The Animation class does not have the correct constructor./r/n" +
                                 "Needs a constructor that takes an Image and a long.", ex);
    }
    catch (IllegalAccessException ex) {
      throw new FactoryException("The Constructor is not accessable in " + animationClass.getName(), ex);
    }
    catch (InstantiationException ex) {
      throw new FactoryException("Unable to create a new Instance of " + animationClass.getName(), ex);
    }
    return animate;
  }

  /**
   * createAnimation
   * The Main Animation creation method, used by the other createAnimation() methods.
   *
   * @param animationClass Class
   * @param durationInMilliseconds int
   * @param clazz Class[]
   * @param imageAry Image[]
   * @return Animation
   * @throws FactoryException
   */
  private Animation createAnimation(Class animationClass, int durationInMilliseconds,
                                    Class[] clazz, Image[] imageAry) throws
      FactoryException {
    Animation animate = null;
    // create the arguments for the constuctor.
    Image arg1 = imageAry[0];
    Object[] args = new Object[] {arg1, new Long(durationInMilliseconds)};
    try {
      // get the constructor
      Constructor constr = animationClass.getConstructor(clazz);
      // instantiate the class.
      animate = (Animation) constr.newInstance(args);
      // add the remaining images.
      for (int i = 1; i < imageAry.length; i++) {
        animate.addFrame(imageAry[i], durationInMilliseconds);
      }
    }
    catch (SecurityException ex) {
      throw new FactoryException("Not enough Security permission to get constructor " + animationClass.getName(), ex);
    }
    catch (NoSuchMethodException ex) {
      throw new FactoryException("The Animation class does not have the correct constructor./r/n" +
                                 "Needs a constructor that takes an Image and a long.", ex);
    }
    catch (InvocationTargetException ex) {
      throw new FactoryException("Unable to create a new Instance of " + animationClass.getName(), ex);
    }
    catch (IllegalArgumentException ex) {
      throw new FactoryException("The Animation class does not have the correct constructor./r/n" +
                                 "Needs a constructor that takes an Image and a long.", ex);
    }
    catch (IllegalAccessException ex) {
      throw new FactoryException("The Constructor is not accessable in " + animationClass.getName(), ex);
    }
    catch (InstantiationException ex) {
      throw new FactoryException("Unable to create a new Instance of " + animationClass.getName(), ex);
    }
    return animate;
  }


  /**
   * createStaticAnimation
   *
   * <p>This creates an animation of the StaticAnimation class.</p>
   *
   * @param image Image
   * @return Animation
   */
  public Animation createStaticAnimation(Image image) {
    Animation animation = new StaticAnimationStrip(image);

    return animation;
  }

  /**
   * createLoopedAnimation
   *
   * @param imageToCut Image
   * @param cellWidth int
   * @param cellHeight int
   * @param numberOfCells int
   * @param durationInMilliseconds int
   * @return Animation
   */
  public Animation createLoopedAnimation(Image imageToCut, int cellWidth,
                                   int cellHeight, int numberOfCells,
                                   int durationInMilliseconds) {
    Image[] imageAry = getImageArray(imageToCut, cellWidth, cellHeight, numberOfCells);
    Animation animation = new LoopedAnimationStrip(imageAry[0], durationInMilliseconds);

    // add the remaining images.
    for (int i = 1; i < imageAry.length; i++) {
      animation.addFrame(imageAry[i], durationInMilliseconds);
    }

    return animation;
  }

  /**
   * createBounceAnimation
   *
   * @param imageToCut Image
   * @param cellWidth int
   * @param cellHeight int
   * @param numberOfCells int
   * @param durationInMilliseconds int
   * @return Animation
   */
  public Animation createBounceAnimation(Image imageToCut, int cellWidth,
                                   int cellHeight, int numberOfCells,
                                   int durationInMilliseconds) {
    Image[] imageAry = getImageArray(imageToCut, cellWidth, cellHeight, numberOfCells);
    Animation animation = new LoopedAnimationStrip(imageAry[0], durationInMilliseconds);

    // add the remaining images.
    for (int i = 1; i < imageAry.length; i++) {
      animation.addFrame(imageAry[i], durationInMilliseconds);
    }
    return animation;
  }

  /**
   * getImageArray
   *
   * @param imageToCut Image
   * @param cellWidth int
   * @param cellHeight int
   * @param numberOfCells int
   * @return Image[]
   */
  private Image[] getImageArray(Image imageToCut, int cellWidth,
                                   int cellHeight, int numberOfCells) {
    Image[] imageAry = imageFactory.getCellsFromImage(imageToCut, cellWidth,
        cellHeight, numberOfCells, DocumentAttributes.IMAGE_TRANSFORM.NORMAL, null);

    return imageAry;
  }

  /**
   * getImageArray
   *
   * @param imageToCut Image
   * @param cellWidth int
   * @param cellHeight int
   * @return Image[]
   */
  private Image[] getImageArray(Image imageToCut, int cellWidth, int cellHeight) {
    Image[] images = imageFactory.getCellsFromImage(imageToCut, cellWidth, cellHeight, DocumentAttributes.IMAGE_TRANSFORM.NORMAL, null);
    return images;
  }
}
