package org.deken.game.animation;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import java.awt.Image;
import org.deken.game.graphics.GameGFX;

/**
 * <p>Title: BounceAnimationStrip</p>
 *
 * <p>Description: An animation strip for a game peice, that bounces from the
 * first frame to the last frame, repeatedly.</p>
 *
 * <p>Copyright: Copyright (c) 2005</p>
 *
 * <p>@author Gary Deken</p>
 * @version 1.0
 */
public class BounceAnimationStrip implements Animation {
   private boolean forward = true;
   private List<AnimatedFrame> frames = new ArrayList<AnimatedFrame>();
   private int totalFrames = 0;
   private int currentFrameIndex = 0;
   // total time of animation.
   private long sequenceTime = 0;
   private int height;
   private int width;
   private int xOffset = 0;
   private int yOffset = 0;
   private String name;

   /**
    * BounceAnimationStrip
    * Used by clone().
    */
   protected BounceAnimationStrip() {
   }

   /**
    * AnimationStrip
    *
    * @param image Image
    * @param duration long
    */
   public BounceAnimationStrip(Image image, long duration) {
      totalFrames = 1;
      frames.add(new AnimatedFrame(image, duration));
      height = image.getHeight(null);
      width = image.getWidth(null);
      reset();
   }

   /**
    * addFrame
    *
    * <p>This adds an image to the animation strip, with a duration in
    * milliseconds.</p>
    *
    * @param image Image
    * @param duration long
    */
   public synchronized void addFrame(Image image, long duration) {
      totalFrames++;
      frames.add(new AnimatedFrame(image, duration));
      if (height == 0 && width == 0) {
         height = image.getHeight(null);
         width = image.getWidth(null);
      }
   }

   /**
    * equals
    *
    * <p>Determines if two animation strips are the same, based on size of image
    * and number of frames.</p>
    *
    * @param obj Object
    * @return boolean
    */
   public boolean equals(Object obj) {
      if (obj instanceof BounceAnimationStrip) {
         BounceAnimationStrip test = (BounceAnimationStrip) obj;
         if (totalFrames == test.totalFrames) {
            if (height == test.height && width == test.width) {
               return true;
            }
         }
      }
      return false;
   }

   /**
    * reset
    *
    * <p>This reset the animation strip, setting it to begin a new.</p>
    */
   public synchronized void reset() {
      currentFrameIndex = 0;
      sequenceTime = 0;
   }

   /**
    * update
    *
    * <p>This takes the elapse time, since the last update, and adds it to the
    * total sequence time. If the duration is exceeded for a
    * frame, the current frame index is increased.</p>
    *
    * @param elapseTime long
    */
   public synchronized void update(long elapseTime) {
      // Add the elapseTime to the sequence time and get the total sequence time.
      sequenceTime += elapseTime;

      // while the sequence time is greater then the frame's time
      while (sequenceTime > getAnimatedFrame(currentFrameIndex).getEndTime()) {
         // get the new sequence time by removing the time of the last frame.
         sequenceTime -= getAnimatedFrame(currentFrameIndex).getEndTime();
         // if the animation is going forward
         if (forward) {
            // if we reach the end of the frames, go back.
            if (currentFrameIndex == totalFrames - 1) {
               currentFrameIndex--;
               forward = false;
            }
            else {
               currentFrameIndex++;
            }
         }
         // if the animation is going backwards.
         else {
            // if we reach the begining, go forward.
            if (currentFrameIndex == 0) {
               currentFrameIndex++;
               forward = true;
            }
            else {
               currentFrameIndex--;
            }
         }
      }
   }

   /**
    * getImage
    *
    * <p>This returns the current frame of the animated strip.</p>
    *
    * @return Image
    */
   public synchronized Image getImage() {
      return getAnimatedFrame(currentFrameIndex).getImage();
   }

   /**
    * Draw
    * Draws the current frame index to the GameGFX, including any animation offsets.
    * @param x
    * @param y
    */
   public void draw(int x, int y) {
      int xL = x + xOffset;
      int yL = y + yOffset;
      GameGFX.gfx().drawImage(getAnimatedFrame(currentFrameIndex).getImage(), xL, yL, null);
   }

   /**
    * getFrame
    *
    * <p>This retrieves the specified frame.</p>
    *
    * @param i int
    * @return AnimatedImageFrame
    */
   public Image getFrame(int i) {
      return frames.get(i).getImage();
   }

   /**
    * getCurrentAnimatedFrame
    *
    * <p>This retrieves the specified Animated Frame.</p>
    *
    * @return AnimatedFrame
    * @param i int
    */
   private AnimatedFrame getAnimatedFrame(int i) {
      return frames.get(i);
   }

   /**
    * getHeight
    *
    * <p>Returns the height of this animation strip.</p>
    *
    * @return int
    */
   public int getHeight() {
      return height;
   }

   /**
    * getWidth
    *
    * <p>Returns the width of this animation strip.</p>
    *
    * @return int
    */
   public int getWidth() {
      return width;
   }

   /**
    * setOffests
    * @param x
    * @param y
    */
   public void setOffsets(int x, int y) {
      xOffset = x;
      yOffset = y;
   }

   /**
    * isAnimating
    *
    * <p>This animation strip will bounce forever. This method will always return
    * true.</p>
    *
    * @return boolean
    */
   public boolean isAnimating() {
      return true;
   }

   /**
    * setName
    * Name is used for debugging.
    *
    * @param name String
    */
   public void setName(String name) {
      this.name = name;
   }

   /**
    * Copy Animation
    * @return BounceAnimationStrip
    */
   public Object clone() {
      BounceAnimationStrip newStrip = new BounceAnimationStrip();
      newStrip.frames = new ArrayList();
      for (Iterator<AnimatedFrame> it = this.frames.iterator(); it.hasNext();) {
         AnimatedFrame animatedFrame = it.next();
         newStrip.frames.add((AnimatedFrame) animatedFrame.clone());
      }
      newStrip.totalFrames = this.totalFrames;
      newStrip.currentFrameIndex = this.currentFrameIndex;
      newStrip.sequenceTime = this.sequenceTime;
      newStrip.height = this.height;
      newStrip.width = this.width;
      newStrip.name = this.name;

      return newStrip;
   }
}
