package org.deken.game.animation;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import java.awt.Image;
import org.deken.game.graphics.GameGFX;

/**
 * <p>Title: BounceOnceAnimationStrip</p>
 *
 * <p>Description: The animation strip runs through the sequence of frames,
 * and upon reaching the end it bounces back, going back to the first frame.
 * At the first frame it stops.</p>
 *
 * <p>Copyright: Copyright (c) 2005</p>
 *
 * <p>@author Gary Deken</p>
 * @version 1.0
 */
public class BounceOnceAnimationStrip implements Animation {
   private boolean forward = true;
   private List<AnimatedFrame> frames = new ArrayList<AnimatedFrame>();
   private int totalFrames = 0;
   private int currentFrameIndex = 0;
   // total time of animation.
   private long sequenceTime = 0;
   private int height;
   private int width;
   private int xOffset = 0;
   private int yOffset = 0;
   private boolean animating;
   private String name;

   /**
    * Used by clone().
    */
   protected BounceOnceAnimationStrip() {
   }

   /**
    * AnimationStrip
    *
    * @param image Image
    * @param duration long
    */
   public BounceOnceAnimationStrip(Image image, long duration) {
      totalFrames = 1;
      frames.add(new AnimatedFrame(image, duration));
      height = image.getHeight(null);
      width = image.getWidth(null);
      reset();
   }

   /**
    * addFrame
    *
    * <p>This adds an image to the animation strip, with a duration in
    * milliseconds.</p>
    *
    * @param image Image
    * @param duration long
    */
   public synchronized void addFrame(Image image, long duration) {
      totalFrames++;
      frames.add(new AnimatedFrame(image, duration));
      if (height == 0 && width == 0) {
         height = image.getHeight(null);
         width = image.getWidth(null);
      }
   }

   /**
    * equals
    *
    * <p>Determines if two animation strips are the same, based on size of image
    * and number of frames.</p>
    *
    * @param obj Object
    * @return boolean
    */
   public boolean equals(Object obj) {
      if (obj instanceof BounceOnceAnimationStrip) {
         BounceOnceAnimationStrip test = (BounceOnceAnimationStrip) obj;
         if (totalFrames == test.totalFrames) {
            if (height == test.height && width == test.width) {
               return true;
            }
         }
      }
      return false;
   }

   /**
    * reset
    *
    * <p>This reset the animation strip, setting it to begin a new.</p>
    */
   public synchronized void reset() {
      currentFrameIndex = 0;
      sequenceTime = 0;
      animating = true;
   }

   /**
    * update
    *
    * <p>This takes the elapse time, since the last update, and adds it to the
    * total sequence time. If the duration is exceeded for a
    * frame, the current frame index is increased.</p>
    *
    * @param elapseTime long
    */
   public synchronized void update(long elapseTime) {
      if (animating) {
         // Add the elapseTime to the sequence time and get the total sequence time.
         sequenceTime += elapseTime;
         // while the elapsed time is greater then the frame's time
         // then set next frame
         while (sequenceTime > getAnimatedFrame(currentFrameIndex).getEndTime()) {
            // get the new sequence time by removing the time of the last frame.
            sequenceTime -= getAnimatedFrame(currentFrameIndex).getEndTime();
            // if the animation is going forward
            if (forward) {
               // if we reach the end of the frames, go back.
               if (currentFrameIndex == totalFrames - 1) {
                  currentFrameIndex--;
                  forward = false;
               }
               else {
                  currentFrameIndex++;
               }
            }
            // if the animation is going backwards.
            else {
               // if we haven't reached the first frame (0), go back.
               // otherwise we stop.
               if (currentFrameIndex > 0) {
                  currentFrameIndex--;
               }
               else {
                  animating = false;
               }
            }
         }
      }
   }

   /**
    * getImage
    *
    * <p>This returns the current frame of the animated strip.</p>
    *
    * @return Image
    */
   public synchronized Image getImage() {
      return getAnimatedFrame(currentFrameIndex).getImage();
   }

   /**
    * Draw
    * Draws the current frame index to the GameGFX, including any animation offsets.
    * @param x
    * @param y
    */
   public void draw(int x, int y) {
      int xL = x + xOffset;
      int yL = y + yOffset;
      GameGFX.gfx().drawImage(getAnimatedFrame(currentFrameIndex).getImage(), xL, yL, null);
   }

   /**
    * getFrame
    *
    * <p>This retrieves the specified frame.</p>
    *
    * @param i int
    * @return AnimatedImageFrame
    */
   public Image getFrame(int i) {
      return frames.get(i).getImage();
   }

   /**
    * getCurrentAnimatedFrame
    *
    * <p>This retrieves the specified Animated Frame.</p>
    *
    * @return AnimatedFrame
    * @param i int
    */
   private AnimatedFrame getAnimatedFrame(int i) {
      return frames.get(i);
   }

   /**
    * getHeight
    *
    * <p>Returns the height of this animation strip.</p>
    *
    * @return int
    */
   public int getHeight() {
      return height;
   }

   /**
    * getWidth
    *
    * <p>Returns the width of this animation strip.</p>
    *
    * @return int
    */
   public int getWidth() {
      return width;
   }

   /**
    * setOffests
    * @param x
    * @param y
    */
   public void setOffsets(int x, int y) {
      xOffset = x;
      yOffset = y;
   }

   /**
    * isAnimating
    *
    * <p>This animation strip will bounce once. Return true until it reaches the
    * end of the bounce cycle, and then return false.</p>
    *
    * @return boolean
    */
   public boolean isAnimating() {
      return animating;
   }

   /**
    * setName
    * Name is used for debugging.
    *
    * @param name String
    */
   public void setName(String name) {
      this.name = name;
   }

   /**
    * Copy Animation
    * @return BounceOnceAnimationStrip
    */
   public Object clone() {
      BounceOnceAnimationStrip newStrip = new BounceOnceAnimationStrip();

      newStrip.frames = new ArrayList();
      for (Iterator<AnimatedFrame> it = this.frames.iterator(); it.hasNext();) {
         AnimatedFrame animatedFrame = it.next();
         newStrip.frames.add((AnimatedFrame) animatedFrame.clone());
      }
      newStrip.animating = this.animating;
      newStrip.totalFrames = this.totalFrames;
      newStrip.currentFrameIndex = this.currentFrameIndex;
      newStrip.sequenceTime = this.sequenceTime;
      newStrip.height = this.height;
      newStrip.width = this.width;
      newStrip.name = this.name;

      return newStrip;
   }
}
