package org.deken.game.animation;

import java.awt.Image;
import org.deken.game.graphics.GameGFX;

/**
 * <p>Title: StaticAnimationStrip</p>
 *
 * <p>Description: An animation strip that is only a single frame.</p>
 *
 * <p>Copyright: Copyright (c) 2005</p>
 *
 * <p>@author Gary Deken</p>
 * @version 1.0
 */
public class StaticAnimationStrip implements Animation {
   private Image image;
   private int height;
   private int width;
   private int xOffset = 0;
   private int yOffset = 0;
   private String name;

   /**
    *
    */
   protected StaticAnimationStrip() {
   }

   public StaticAnimationStrip(Image image, long lon) {
      height = image.getHeight(null);
      width = image.getWidth(null);
      this.image = image;
   }

   /**
    * Animation Strip
    * @param image
    */
   public StaticAnimationStrip(Image image) {
      height = image.getHeight(null);
      width = image.getWidth(null);
      this.image = image;
   }

   /**
    * getImage
    *
    * @return Image
    */
   public Image getImage() {
      return image;
   }

   /**
    * Draw
    * Draws the current frame index to the GameGFX, including any animation offsets.
    * @param x
    * @param y
    */
   public void draw(int x, int y) {
      int xL = x + xOffset;
      int yL = y + yOffset;
      GameGFX.gfx().drawImage(image, xL, yL, null);
   }

   /**
    * addFrame
    *
    * <p>This method does nothing, since the animation strip, has only a single
    * image.</p>
    *
    * @param image Image
    * @param duration long
    */
   public void addFrame(Image image, long duration) {
      this.image = image;
      if (height == 0 && width == 0) {
         height = image.getHeight(null);
         width = image.getWidth(null);
      }
   }

   /**
    * equals
    *
    * <p>Determines if two animation strips are the same, based on size of image
    * and number of frames.</p>
    *
    * @param obj Object
    * @return boolean
    */
   public boolean equals(Object obj) {
      if (obj instanceof StaticAnimationStrip) {
         StaticAnimationStrip test = (StaticAnimationStrip) obj;
         if (height == test.height && width == test.width) {
            return true;
         }
      }
      return false;
   }

   /**
    * reset
    *
    * <p>This method does nothing, since there is only a single image, there is
    * nothing to reset.</p>
    */
   public void reset() {
   }

   /**
    * update
    *
    * <p>This method does nothing, with the static animation strip, the same
    * image is always returned.</p>
    *
    * @param elapseTime long
    */
   public void update(long elapseTime) {
   }

   /**
    * getFrame
    *
    * <p>This method returns the only image in the animation.</p>
    *
    * @param i int
    * @return Image
    */
   public Image getFrame(int i) {
      return image;
   }

   /**
    * getHeight
    *
    * <p>Returns the height of this animation strip.</p>
    *
    * @return int
    */
   public int getHeight() {
      return height;
   }

   /**
    * getWidth
    *
    * <p>Returns the width of this animation strip.</p>
    *
    * @return int
    */
   public int getWidth() {
      return width;
   }

   /**
    * setOffests
    * @param x
    * @param y
    */
   public void setOffsets(int x, int y) {
      xOffset = x;
      yOffset = y;
   }

   /**
    * isAnimating
    *
    * <p>This animation strip does not animate, always returning the same
    * image.</p>
    *
    * @return boolean
    */
   public boolean isAnimating() {
      return false;
   }

   /**
    * setName
    * Name is used for debugging.
    *
    * @param name String
    */
   public void setName(String name) {
      this.name = name;
   }

   /**
    * Copy Animation
    * @return StaticAnimationStrip
    */
   public Object clone() {
      StaticAnimationStrip newStrip = new StaticAnimationStrip();

      newStrip.image = this.image;
      newStrip.height = this.height;
      newStrip.width = this.width;
      newStrip.name = this.name;

      return newStrip;
   }
}
